# encoding: utf-8
from collections import OrderedDict

from efa.views.participation import create_oi
from pyramid.view import view_config
from ringo.lib.helpers import (
    get_action_routename, get_item_modul, dynamic_import
)
from ringo.lib.renderer import InfoDialogRenderer
from ringo.views.base import create, delete
from ringo.views.helpers import get_item_from_request
from ringo.views.request import get_return_value, is_confirmed
from ringo_printtemplate.lib.renderer import PrintDialogRenderer
from ringo_printtemplate.views import print_template, retrieve_print_template
from waskiq.model.angebot import Angebot
from waskiq.model.baustein_cm import BausteinCM


@view_config(route_name=get_action_routename(BausteinCM, 'print'),
             renderer="/default/print.mako",
             permission='read',
             request_method="POST"
             )
def _print(request):
    """Custom Print method for participants. This custom print function
    is a hack to add the organisat:ion and project value for the current
    participant."""
    item = get_item_from_request(request)
    renderer = PrintDialogRenderer(request, item)
    form = renderer.form
    if (request.method == 'POST' and
       is_confirmed(request) and
       form.validate(request.params)):
        foerderplan = generate_foerderplan(request)
        template_id = request.POST['printtemplates']
        template = retrieve_print_template(request, template_id)
        return print_template(request, foerderplan, template)
    else:
        clazz = item.__class__
        rvalue = {}
        rvalue['dialog'] = renderer.render()
        rvalue['clazz'] = clazz
        rvalue['item'] = item
        return rvalue


def generate_foerderplan(request):
    baustein = get_item_from_request(request)
    angebote = retrieve_angebote_from_db(baustein, request)
    foerderplan = {}
    foerderplan['participation'] = baustein.participation
    foerderplan['owner'] = baustein.owner
    foerderplan['angebote'] = group_by_bereich(angebote)
    foerderplan['german_date'] = format_date(baustein.datum)
    return foerderplan


def retrieve_angebote_from_db(item, request):
    angebote = request.db.query(Angebot) \
        .filter(Angebot.baustein_cm_id == item.id) \
        .order_by(Angebot.bereich, Angebot.von).all()
    return angebote


def group_by_bereich(angebote):
    angebote_per_bereich = OrderedDict({})
    for angebot in angebote:
        if angebot.bereich not in angebote_per_bereich:
            angebote_per_bereich[angebot.bereich] = {}
            bereich_name = angebot.get_value('bereich', expand=True)
            name = bereich_name if bereich_name else 'NN'
            angebote_per_bereich[angebot.bereich]['name'] = name
            angebote_per_bereich[angebot.bereich]['data'] = []
        angebote_per_bereich[angebot.bereich]['data'].append(generate(angebot))
    return angebote_per_bereich


def format_date(datum):
    result = datum
    try:
        result = datum.strftime('%d.%m.%Y')
    except:
        pass
    return result


def format_institution(institution):
    return institution.replace('\r', '').split('\n')


def generate(angebot):
    return {'sonstiges1': format_sonstiges(angebot.sonstiges1),
            'benennung': angebot.benennung,
            'institution': format_institution(angebot.institution),
            'german_von': format_date(angebot.von),
            'german_bis': format_date(angebot.bis),
            'ziel': angebot.ziel,
            'ziel1': angebot.ziel1,
            'ziel2': angebot.ziel2,
            'ziel3': angebot.ziel3,
            'ziel4': angebot.ziel4,
            'ziel5': angebot.ziel5,
            }


def format_sonstiges(sonstiges):
    if sonstiges:
        return sonstiges+' - '
    return sonstiges


def _is_last_baustein(request):
    baustein = get_item_from_request(request)
    return len(baustein.participation.baustein) < 2


# TODO: Delete this method here in import it from ringo.views.helpers
# (ti) <2015-11-26 15:40>
def get_related_item_from_request(request):
    params = request.params.get("addrelation")
    clazz = params.split(":")[1]
    item_id = params.split(":")[2]
    factory = dynamic_import(clazz).get_item_factory(request)
    return factory.load(item_id)


def _is_max_baustein(request):

    # The creation is only needed here to be able to get a dummy item
    # from the request.
    clazz = request.context.__model__
    try:
        factory = clazz.get_item_factory(request)
    except TypeError:
        # Old version of get_item_factory method which does not take an
        # request parameter.
        factory = clazz.get_item_factory()
        factory._request = request
    if not request.context.item:
        request.context.item = factory.create(request.user, {})

    item = get_item_from_request(request)
    tblname = item.__class__.__tablename__
    participation = get_related_item_from_request(request)
    if tblname == "baustein_clearings":
        return len(participation.baustein_clearings) > 0
    elif tblname == "baustein_jsas":
        return len(participation.baustein_jsas) > 0
    elif tblname == "baustein_cms":
        return len(participation.baustein_cms) > 0
    elif tblname == "baustein_mps":
        return len(participation.baustein_mps) > 0
    return True


def _render_info_last_baustein(request):
    rvalues = get_return_value(request)
    #_ = request.translate
    #title = _("Cannot delete the last brick")
    #body = _("At least one brick must be in a participation. So you cannot"
    #         " delete the last brick.")
    #renderer = InfoDialogRenderer(request, title=title, body=body)
    #rvalues['dialog'] = renderer.render([])
    item = get_item_from_request(request)
    modul = get_item_modul(request, item)
    title = u"Der Baustein kann nicht gelöscht werden"
    body = (u"Der „%s“ in der Teilnahme von „%s“ ist der einzige Baustein in "
            u"der Teilnahme und kann daher nicht gelöscht werden."
            % (modul.label, item))
    dialog = InfoDialogRenderer(request, title, body)
    rvalues['dialog'] = dialog.render()
    return rvalues


def _render_info_max_baustein(request):
    participation = get_related_item_from_request(request)
    rvalues = get_return_value(request)
    title = u"Der Baustein kann nicht angelegt werden"
    body = (u"Es existiert bereits ein Baustein diesen Typs in der"
            u" Teilnahme von „%s“." % (participation))
    dialog = InfoDialogRenderer(request, title, body)
    rvalues['form'] = dialog.render()
    return rvalues


@view_config(route_name=get_action_routename(BausteinCM, 'create'),
             renderer="/default/create.mako",
             permission='create'
             )
def create_(request):
    if _is_max_baustein(request):
        return _render_info_max_baustein(request)
    rvalue = create(request)

    #Check if OI must be added.
    clazz = request.context.__model__
    participation = get_related_item_from_request(request)
    if request.POST and clazz is BausteinCM and not participation.participant.oi:
        create_oi(request.user, participation)
    return rvalue


@view_config(route_name=get_action_routename(BausteinCM, 'delete'),
             renderer="/default/confirm.mako",
             permission='delete'
             )
def delete_(request):
    if _is_last_baustein(request):
        return _render_info_last_baustein(request)
    return delete(request)
