# -*- coding: utf8 -*-
from pyramid.view import view_config
from pyramid.renderers import render_to_response
from formbar.form import Form, FormDataprovider
from ringo.lib.form import get_form_config_from_file, get_eval_url
from ringo.lib.helpers import get_app_url
from ringo.lib.i18n import locale_negotiator
from ringo.model.user import User
from efa.model.project import Project
from efa.model.organisation import Organisation
from waskiq.model.participation import WaskiqParticipation as Participation
from efa.views.export_participations import (
    access_allowed, build_response, get_metadata
)
from efa.lib.excelexport import QueryData


class WaskiqQueryData(QueryData):
    """WaskiqQueryData can handle additional filtering after projects and
       organisations."""
    def __init__(self,
                 filter_type,
                 start,
                 end,
                 gender,
                 projects,
                 organisations):
        self.filter_type = filter_type
        self.start = start
        self.end = end
        self.gender = gender
        self.projects = projects  # sequence of integers for ids
        self.organisations = organisations
        self.foerderphase = None

    def filter(self, query, clazz):
        """We want additional filtering here. The other filtering is done
        thru the super filter."""
        result_query = super(WaskiqQueryData, self).filter(query, clazz)

        if self.projects:
            result_query = result_query.filter(clazz.project_id.in_(
                self.projects))

        if self.organisations:
            result_query = result_query.join(clazz.owner) \
                .join(Organisation, User.organisation) \
                .filter(Organisation.id.in_(self.organisations))

        return result_query


class ExcelexportformDataprovider(FormDataprovider):
    """Helper class for inserting the options for projects and organisation
    into the form. We give it a request as ctx."""

    @property
    def projects_options(self):
        return sorted([(unicode(x), str(x.id))
                      for x in self.ctx.db.query(Project).all()])

    @property
    def organisations_options(self):
        return sorted([(unicode(x), str(x.id))
                      for x in self.ctx.db.query(Organisation).all()])


def extract_query_data_from_form(form):
    """Extract all data from the form."""
    return WaskiqQueryData(form.data.get("filter_type"),
                           form.data.get("start_date"),
                           form.data.get("end_date"),
                           form.data.get("gender"),
                           form.data.get("projects"),
                           form.data.get("organisations"))


def prepare_form(request):
    """Load the form from waskiq and add the dataprovider to inject the
    options for projects and organisations."""
    formconfig = get_form_config_from_file("waskiq", "excelexport-waskiq.xml",
                                           "entryandexit")

    form = Form(formconfig, ExcelexportformDataprovider(request), request.db,
                translate=request.translate,
                renderers={},
                change_page_callback={'url': 'set_current_form_page'},
                locale=locale_negotiator(request),
                url_prefix=get_app_url(request),
                eval_url=get_eval_url(),
                csrf_token=request.session.get_csrf_token())
    return form


def get_waskiq_metadata(request, form):
    metadata = get_metadata(request, form)
    METADATA_WASKIQ_EXTRA_FIELDS = ['organisations', 'projects']
    for field_str in METADATA_WASKIQ_EXTRA_FIELDS:
        field = form.get_field(field_str)
        field.value = form.data[field_str]
        # we call expand_value directly, because get_value of Field changes
        # value into a string. (Workaround)
        value = field.expand_value(form.data[field_str])
        metadata.append([request.translate(field.label), value])
    return metadata


@view_config(route_name='participations-excelexport', request_method='POST')
def participations_excel_export_waskiq(request):
    access_allowed(request.user)

    form = prepare_form(request)
    form.validate(request.POST)

    if not form.has_errors():
        query_data = extract_query_data_from_form(form)
        metadata = get_waskiq_metadata(request, form)
        return build_response(request, query_data, Participation, "waskiq",
                              metadata)
    else:
        return render_to_response('/excelexportform.mako',
                                  {'form': form,
                                   'rendered_form': form.render(),
                                   'exportname': 'Teilnahmen'},
                                  request)


@view_config(route_name='participations-excelexport',
             renderer='/excelexportform.mako',
             request_method='GET')
def participations_excelexport_form(request):
    access_allowed(request.user)
    form = prepare_form(request)
    return {'form': form, 'rendered_form': form.render(),
            'exportname': 'Teilnahmen'}
