# encoding: utf-8
import logging

from efa.lib.validators import (
    euhu_validator_factory,
    build_data_to_validate
)
from efa.model.participation import calc_complete
from efa.views.participation import (
    create_oi,
    update_ as efa_update,
    create_ as efa_create,
    generate_controllist,
    check_and_set_fp1_personal_id)
from euhu.programs.waskiq import validate
from pyramid.httpexceptions import HTTPUnauthorized
from pyramid.view import view_config
from ringo.lib import security
from ringo.lib.helpers import get_action_routename
from ringo.views.helpers import (
    render_item_form,
    get_item_from_request,
    get_item_form,
    get_item_modul
)
from ringo.views.request import (
    handle_redirect_on_success,
    get_return_value,
)
from waskiq.lib.renderer import BausteinRenderer
from waskiq.lib.validators import check_oi_validator_factory
from waskiq.model.baustein_clearing import BausteinClearing
from waskiq.model.baustein_cm import BausteinCM
from waskiq.model.baustein_jsa import BausteinJSA
from waskiq.model.baustein_mp import BausteinMP
from waskiq.model.participation import (
    WaskiqParticipation, MIKROPROJECT_ID, CM_ID, CLEARING_ID, JSA_ID
)
log = logging.getLogger(__name__)


# We don't move this validator into the form, because it a special case
# of validators (global, warning) which are not  supported in formbar.
# See issue3148.
def euhu_validator(field, data, request):
    participation = get_item_from_request(request)
    # For GET request return the cached validation state.
    if request.method == "GET" and hasattr(participation, "valid"):
        if participation.valid is not None:
            return participation.valid

    # Save the result of the validation.
    if hasattr(participation, "valid"):
        # Revalidate the participation
        data_factory = build_data_to_validate(data, participation)

        if validate(data_factory).has_violated_rules():
            participation.valid = 0
            return False
        else:
            participation.valid = 1
            return True
    return True


def add_baustein(item, baustein_id, request):
    user = request.user
    if baustein_id == MIKROPROJECT_ID:
        cls = BausteinMP
    elif baustein_id == CM_ID:
        cls = BausteinCM
    elif baustein_id == CLEARING_ID:
        cls = BausteinClearing
    elif baustein_id == JSA_ID:
        cls = BausteinJSA
    else:
        raise ValueError("Wrong type of baustein")
    factory = cls.get_item_factory()
    baustein = factory.create(user=user, values={})
    baustein.participation = item
    baustein.save({}, request)
    item_label = get_item_modul(request, cls).get_label()
    log_msg = u'User {user.login} created {item_label} {item.id}' \
        .format(item_label=item_label, item=baustein, user=request.user)
    log.info(log_msg)


def set_oi_status(request):
    item = get_item_from_request(request)
    if item.waskiqoi is not None:
        item.waskiqoi.status = None


def set_complete(request):
    item = get_item_from_request(request)
    item.complete = calc_complete(item)


def _render_baustein_form(request, form=None):
    if not form:
        form = get_item_form('baustein', request)
    values = {'_roles': [str(r.name) for r in request.user.roles]}
    values.update(request.ringo.params.values)
    rvalues = get_return_value(request)
    rvalues['form'] = render_item_form(request, form, values, False)
    return rvalues


@view_config(route_name=get_action_routename(WaskiqParticipation, 'create'),
             renderer='/default/create.mako',
             permission='create')
def create_(request):
    return efa_create(request, external_callbacks=[check_and_set_fp1_personal_id])


@view_config(route_name=get_action_routename(WaskiqParticipation, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request):
    if request.method == 'GET':
        request.session['baustein_post'] = None
    participation = get_item_from_request(request)
    if not request.POST and not participation.baustein:
        return _render_baustein_form(request)
    elif request.POST and not participation.baustein:
        form = get_item_form('baustein', request)
        if form.validate(request.params):
            if "baustein" in form.data:
                oi = None
                if form._item.participant:
                    oi = form._item.participant.oi

                # For newly created participation, add OI
                if not oi:
                    create_oi(request.user, form._item)
                for baustein in form.data['baustein']:
                    if baustein is not None:
                        add_baustein(form._item, baustein, request)
            set_complete(request)
            return handle_redirect_on_success(request)
        else:
            set_complete(request)
            return _render_baustein_form(request, form)
    elif request.POST and participation.baustein:
        set_oi_status(request)
    validators = [euhu_validator_factory(request, euhu_validator),
                  check_oi_validator_factory(request)]

    renderers = {"baustein": BausteinRenderer}
    set_complete(request)
    return efa_update(request, validators, renderers=renderers)


@view_config(route_name="controllist",
             renderer="/default/list.mako")
def controllist(request):
    """Display a summary list of participations for organisation users."""
    if not security.has_role(request.user, "beneficiary"):
        raise HTTPUnauthorized
    return generate_controllist(request, WaskiqParticipation)
